package org.apache.sandesha2.policy.processors;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.sandesha2.policy.PolicyEngineData;
import org.apache.sandesha2.policy.RMPolicy;
import org.apache.sandesha2.policy.RMPolicyToken;
import org.apache.sandesha2.policy.RMProcessorContext;

public class StorageManagersProcessor {
	private boolean initializedStorageManager = false;

	private Log logger = LogFactory.getLog(this.getClass().getName());

	public void initializeStorageManager(RMPolicyToken rmpt)
			throws NoSuchMethodException {
		RMPolicyToken tmpRpt = RMPolicy.storageManager.copy();
		tmpRpt.setProcessTokenMethod(this);
		rmpt.setChildToken(tmpRpt);

		tmpRpt = RMPolicy.permenentStorageManager.copy();
		tmpRpt.setProcessTokenMethod(this);
		rmpt.setChildToken(tmpRpt);

	}

	public Object doStorageManagers(RMProcessorContext rmpc) {
		RMPolicyToken rmpt = rmpc.readCurrentSecurityToken();
		switch (rmpc.getAction()) {

		case RMProcessorContext.START:
			if (!initializedStorageManager) {
				try {
					initializeStorageManager(rmpt);
					initializedStorageManager = true;
				} catch (NoSuchMethodException e) {
					logger.error(
							"Exception occured in initializeStorageManager", e);
					return new Boolean(false);
				}
			}
			logger.debug(rmpt.getTokenName());

		case RMProcessorContext.COMMIT:
			break;
		case RMProcessorContext.ABORT:
			break;
		}
		return new Boolean(true);
	}

	public Object doStorageManager(RMProcessorContext rmpc) {
		PolicyEngineData ped = rmpc.readCurrentPolicyEngineData();
		String cls = rmpc.getAssertion().getStrValue();

		if (cls != null && !cls.trim().equals("")) {
			ped.setStorageManager(cls.trim());
		}

		return new Boolean(true);
	}

	public Object doPermanentStorageManager(RMProcessorContext spc) {
		PolicyEngineData ped = spc.readCurrentPolicyEngineData();
		String cls = spc.getAssertion().getStrValue();

		if (cls != null && !cls.trim().equals("")) {
			ped.setStorageManager(cls.trim());
		}

		return new Boolean(true);
	}
}
